searchData={"content_type":"text/markdown","items":[{"doc":"`AppIdentity` is an Elixir implementation of the Kinetic Commerce application\n[identity proof algorithm](spec.md).\n\nIt implements identity proof generation and validation functions. These\nfunctions expect to work with an application struct (`t:AppIdentity.App.t/0`).","ref":"AppIdentity.html","title":"AppIdentity","type":"module"},{"doc":"If [telemetry](https://hexdocs.pm/telemetry) is a dependency in your\napplication, and the telemetry is not explicitly disabled, telemetry events\nwill be emitted for `AppIdentity.generate_proof/2`,\n`AppIdentity.verify_proof/3`, and `AppIdentity.Plug`. See\n`AppIdentity.Telemetry` for more information.\n\n#","ref":"AppIdentity.html#module-telemetry-support","title":"Telemetry Support - AppIdentity","type":"module"},{"doc":"Telemetry may be disabled by setting this for your configuration:\n\n    config :app_identity, AppIdentity.Telemetry, enabled: false\n\nRecompile `app_identity` if this setting is changed.","ref":"AppIdentity.html#module-disabling-telemetry","title":"Disabling Telemetry - AppIdentity","type":"module"},{"doc":"Generate an identity proof string for the given application. Returns `{:ok,\nproof}` or `:error`.\n\nIf `nonce` is provided, it must conform to the shape expected by the proof\nversion. If not provided, it will be generated.\n\nIf `version` is provided, it will be used to generate the nonce and the proof.\nThis will allow a lower level application to raise its version level.\n\n#","ref":"AppIdentity.html#generate_proof/2","title":"AppIdentity.generate_proof/2","type":"function"},{"doc":"A version 1 app can have a fixed nonce, which will always produce the same\nvalue.\n\n    iex> {:ok, app} = AppIdentity.App.new(%{version: 1, id: \"decaf\", secret: \"bad\"})\n    iex> AppIdentity.generate_proof(app, nonce: \"hello\")\n    {:ok, \"ZGVjYWY6aGVsbG86RDNGNjJCQTYyOEIyMzhEOTgwM0MyNEU4NkNCOTY3M0ZEOTVCNTdBNkJGOTRFMkQ2NTMxQTRBODg1OTlCMzgzNQ==\"}\n\nA version 2 app fails when given a non-timestamp nonce.\n\n    iex> AppIdentity.generate_proof(v1(), version: 2, nonce: \"hello\")\n    :error\n\nA version 2 app _cannot_ generate a version 1 nonce.\n\n    iex> AppIdentity.generate_proof(v2(), version: 1)\n    :error\n\nA version 2 app will be rejected if the version has been disallowed.\n\n    iex> AppIdentity.generate_proof(v2(), disallowed: [1, 2])\n    :error\n\n#","ref":"AppIdentity.html#generate_proof/2-examples","title":"Examples - AppIdentity.generate_proof/2","type":"function"},{"doc":"When telemetry is enabled, `generate_proof/2` will emit:\n\n- `[:app_identity, :generate_proof, :start]`\n- `[:app_identity, :generate_proof, :stop]`","ref":"AppIdentity.html#generate_proof/2-telemetry","title":"Telemetry - AppIdentity.generate_proof/2","type":"function"},{"doc":"Generate an identity proof string for the given application. Returns the proof\nstring or raises an exception on error.\n\nIf `nonce` is provided, it must conform to the shape expected by the proof\nversion. If not provided, it will be generated.\n\nIf `version` is provided, it will be used to generate the nonce and the proof.\nThis will allow a lower level application to raise its version level.\n\n#","ref":"AppIdentity.html#generate_proof!/2","title":"AppIdentity.generate_proof!/2","type":"function"},{"doc":"A version 1 app can have a fixed nonce, which will always produce the same\nvalue.\n\n    iex> {:ok, app} = AppIdentity.App.new(%{version: 1, id: \"decaf\", secret: \"bad\"})\n    iex> AppIdentity.generate_proof!(app, nonce: \"hello\")\n    \"ZGVjYWY6aGVsbG86RDNGNjJCQTYyOEIyMzhEOTgwM0MyNEU4NkNCOTY3M0ZEOTVCNTdBNkJGOTRFMkQ2NTMxQTRBODg1OTlCMzgzNQ==\"\n\nA version 2 app fails when given a non-timestamp nonce.\n\n    iex> AppIdentity.generate_proof!(v1(), version: 2, nonce: \"hello\")\n    ** (AppIdentity.AppIdentityError) Error generating proof\n\nA version 2 app _cannot_ generate a version 1 nonce.\n\n    iex> AppIdentity.generate_proof!(v2(), version: 1)\n    ** (AppIdentity.AppIdentityError) Error generating proof\n\nA version 2 app will be rejected if the version has been disallowed.\n\n    iex> AppIdentity.generate_proof!(v2(), disallowed: [1, 2])\n    ** (AppIdentity.AppIdentityError) Error generating proof\n\n#","ref":"AppIdentity.html#generate_proof!/2-examples","title":"Examples - AppIdentity.generate_proof!/2","type":"function"},{"doc":"When telemetry is enabled, `generate_proof!/2` will emit:\n\n- `[:app_identity, :generate_proof, :start]`\n- `[:app_identity, :generate_proof, :stop]`\n\nTelemetry events are emitted *before* any error exceptions are thrown.","ref":"AppIdentity.html#generate_proof!/2-telemetry","title":"Telemetry - AppIdentity.generate_proof!/2","type":"function"},{"doc":"The name, version, and supported specification version of this App Identity\npackage for Elixir.","ref":"AppIdentity.html#info/0","title":"AppIdentity.info/0","type":"function"},{"doc":"The name, version, or supported specification version of this App Identity\npackage for Elixir.","ref":"AppIdentity.html#info/1","title":"AppIdentity.info/1","type":"function"},{"doc":"Parses a proof string into an AppIdentity.Proof struct. Returns `{:ok, proof}`\nor `:error`.","ref":"AppIdentity.html#parse_proof/1","title":"AppIdentity.parse_proof/1","type":"function"},{"doc":"Parses a proof string into an AppIdentity.Proof struct. Returns the parsed\nproof or raises an exception.","ref":"AppIdentity.html#parse_proof!/1","title":"AppIdentity.parse_proof!/1","type":"function"},{"doc":"Verify a `AppIdentity` proof value using a a provided `app`. Returns `{:ok,\napp}`, `{:ok, nil}`, or `:error`.\n\nThe `proof` may be provided either as a string or a parsed\n`t:AppIdentity.Proof.t/0` (from `parse_proof/1`). String proof values are\nusually obtained from HTTP headers. At Kinetic Commerce, this has generally\njeen `KCS-Application` or `KCS-Service`.\n\nThe `app` can be provided as one of `t:AppIdentity.App.input/0`,\n`t:AppIdentity.App.t/0`, or `t:AppIdentity.App.finder/0`. If provided\na finder, it will be called with the `proof` value.\n\n`verify_proof/3` has three possible return values:\n\n- `{:ok, AppIdentity.App}` when the proof is validated against the provided or\n  located application;\n- `{:ok, nil}` when the proof matches the provided or located application, but\n  it does not validate.\n- `:error` when there is any error during proof validation.\n\n```elixir\nAppIdentity.verify_proof(proof, &IdentityApplications.get!(&1.id))\n```\n\n#","ref":"AppIdentity.html#verify_proof/3","title":"AppIdentity.verify_proof/3","type":"function"},{"doc":"When telemetry is enabled, `verif_proof/3` will emit:\n\n- `[:app_identity, :verify_proof, :start]`\n- `[:app_identity, :verify_proof, :stop]`","ref":"AppIdentity.html#verify_proof/3-telemetry","title":"Telemetry - AppIdentity.verify_proof/3","type":"function"},{"doc":"Verify a `AppIdentity` proof value using a a provided `app`. Returns the app,\nnil, or raises an exception on error.\n\nThe `proof` may be provided either as a string or a parsed\n`t:AppIdentity.Proof.t/0` (from `parse_proof/1`). String proof values are\nusually obtained from HTTP headers. At Kinetic Commerce, this has generally\njeen `KCS-Application` or `KCS-Service`.\n\nThe `app` can be provided as one of `t:AppIdentity.App.input/0`,\n`t:AppIdentity.App.t/0`, or `t:AppIdentity.App.finder/0`. If provided\na finder, it will be called with the `proof` value.\n\n`verify_proof/3` has two possible return values:\n\n- `AppIdentity.App` when the proof is validated against the provided or\n  located application;\n- `nil` when the proof matches the provided or located application, but it\n  does not validate.\n\nIt raises an exception on any error during proof validation.\n\n```elixir\nAppIdentity.verify_proof(proof, &IdentityApplications.get!(&1.id))\n```\n\n#","ref":"AppIdentity.html#verify_proof!/3","title":"AppIdentity.verify_proof!/3","type":"function"},{"doc":"When telemetry is enabled, `verify_proof!/3` will emit:\n\n- `[:app_identity, :verify_proof, :start]`\n- `[:app_identity, :verify_proof, :stop]`\n\nTelemetry events are emitted *before* any error exceptions are thrown.","ref":"AppIdentity.html#verify_proof!/3-telemetry","title":"Telemetry - AppIdentity.verify_proof!/3","type":"function"},{"doc":"A list of algorithm versions that are not allowed.\n\nThe presence of an app in this list will prevent the generation or\nverification of proofs for the specified version.\n\nIf `nil`, an empty list, or missing, all versions are allowed.","ref":"AppIdentity.html#t:disallowed/0","title":"AppIdentity.disallowed/0","type":"type"},{"doc":"The App Identity app unique identifier. Validation of the `id` value will\nconvert non-string IDs using Kernel.to_string/1.\n\nIf using integer IDs, it is recommended that the `id` value be provided as\nsome form of extended string value, such as that provided by Rails [global\nID](https://github.com/rails/globalid) or the `absinthe_relay`\n[Node.IDTranslator](https://hexdocs.pm/absinthe_relay/Absinthe.Relay.Node.IDTranslator.html).\nSuch representations are _also_ recommended if the ID is a compound value.\n\n`t:id/0` values _must not_ contain a colon (`:`) character.","ref":"AppIdentity.html#t:id/0","title":"AppIdentity.id/0","type":"type"},{"doc":"A nonce value used in the algorithm proof. The shape of the nonce depends on\nthe algorithm `t:version/0`.\n\nVersion 1 `t:nonce/0` values should be cryptographically secure and\nnon-sequential, but sufficiently fine-grained timestamps (those including\nmicroseconds, as `yyyymmddHHMMSS.sss`) _may_ be used. Version 1 proofs verify\nthat the nonce is at least one byte long and do not contain a colon (`:`).\n\nVersion 2, 3, and 4 `t:nonce/0` values only permit fine-grained timestamps\nthat should be generated from a clock in sync with Network Time Protocol\nservers. The timestamp will be parsed and compared to the server time (also in\nsync with an NTP server).","ref":"AppIdentity.html#t:nonce/0","title":"AppIdentity.nonce/0","type":"type"},{"doc":"Options for generating or verifying proofs.\n\n- `nonce` can specify a precomputed nonce for proof generation. It will be\n  verified by the algorithm version for correctness and compatibility, but\n  will otherwise be used unmodified. This option is ignored for proof\n  verification.\n\n- `version` can specify the generation of a proof compatible with, but\n  different than, the application version. This option is ignored for proof\n  verification.","ref":"AppIdentity.html#t:option/0","title":"AppIdentity.option/0","type":"type"},{"doc":"The App Identity app secret value. This value is used _as provided_ with no\nencoding or decoding. Because this is a sensitive value, it may be provided as\na closure in line with the EEF Security Working Group sensitive data\n[recommendation](https://erlef.github.io/security-wg/secure_coding_and_deployment_hardening/sensitive_data#wrapping).\n\n`AppIdentity.App` always stores this value as a closure.","ref":"AppIdentity.html#t:secret/0","title":"AppIdentity.secret/0","type":"type"},{"doc":"The positive integer version of the App Identity algorithm to use. Will be\nvalidated to be a supported version for app creation, and not an explicitly\ndisallowed version during proof validation.\n\nIf provided as a string value, it must convert cleanly to an integer value,\nwhich means that a version of `\"3.5\"` is not a valid value.\n\nApp Identity algorithm versions are strictly upgradeable. That is, a version\n1 app can verify version 1, 2, 3, or 4 proofs. However, a version 2 app will\n_never_ validate a version 1 proof.\n\n \n   \n     \n       App Identity Version \n       Nonce \n       Digest Algorithm \n       Can Verify Proof from Version \n     \n      1  2  3  4  \n   \n   \n     \n       1 \n       random \n       SHA2-256 \n       Yes \n       Yes \n       Yes \n       Yes \n     \n     \n       2 \n       timestamp ± fuzz \n       SHA2-256 \n       - \n       Yes \n       Yes \n       Yes \n     \n     \n       3 \n       timestamp ± fuzz \n       SHA2-384 \n       - \n       - \n       Yes \n       Yes \n     \n     \n       4 \n       timestamp ± fuzz \n       SHA2-512 \n       - \n       - \n       - \n       Yes","ref":"AppIdentity.html#t:version/0","title":"AppIdentity.version/0","type":"type"},{"doc":"The struct used by the App Identity proof generation and verification\nalgorithms. This should be constructed from a provided map or struct, such as\na static configuration file or a database record.\n\nThe original app input is stored in the `source` attribute.","ref":"AppIdentity.App.html","title":"AppIdentity.App","type":"module"},{"doc":"Converts the provided `t:input/0` value into an App struct (`t:t/0`). May be\nprovided a `t:loader/0` function that will return an `t:input/0` value for\nvalidation and conversion.\n\nIf provided an App struct (`t:t/0`), returns the provided app struct without\nvalidation.\n\nThe result is `{:ok, app}` or `{:error, reason}`.\n\n#","ref":"AppIdentity.App.html#new/1","title":"AppIdentity.App.new/1","type":"function"},{"doc":"iex> App.new(%{})\n    {:error, \"id must not be nil\"} = App.new(%{id: nil})\n\n    iex> {:ok, app} = App.new(%{id: 1, secret: \"secret\", version: 1})\n    iex> app.source\n    %{id: 1, secret: \"secret\", version: 1}\n    iex> app.secret.()\n    \"secret\"\n\n    iex> {:ok, app} = App.new(%{id: 1, secret: \"secret\", version: 1})\n    iex> {:ok, app_copy} = App.new(app)\n    iex> app == app_copy\n    true","ref":"AppIdentity.App.html#new/1-examples","title":"Examples - AppIdentity.App.new/1","type":"function"},{"doc":"An optional configuration value for validation of an `AppIdentity` proof.\n\nIf not provided, the default value when required is `{fuzz: 600}`, specifying\nthat the timestamp may not differ from the current time by more than ±600\nseconds (±10 minutes). Depending on the nature of the app being verified and\nthe expected network conditions, a shorter time period than 600 seconds is\nrecommended.\n\nThe `AppIdentity` version 1 algorithm does not use `config`.","ref":"AppIdentity.App.html#t:config/0","title":"AppIdentity.App.config/0","type":"type"},{"doc":"A finder function accepting a Proof struct parameter that returns a map or\nstruct that can be converted into an App struct.","ref":"AppIdentity.App.html#t:finder/0","title":"AppIdentity.App.finder/0","type":"type"},{"doc":"A map or struct that can be converted into an App struct. If the map uses\nstring keys, they are required to match the same definitions.","ref":"AppIdentity.App.html#t:input/0","title":"AppIdentity.App.input/0","type":"type"},{"doc":"","ref":"AppIdentity.App.html#t:invalid_input/0","title":"AppIdentity.App.invalid_input/0","type":"type"},{"doc":"A 0-arity loader function that returns a map or struct that can be converted\ninto an App struct.","ref":"AppIdentity.App.html#t:loader/0","title":"AppIdentity.App.loader/0","type":"type"},{"doc":"A representation of an AppIdentity app used for proof generation and\nverification.\n\nThe `t:input/0` value used in the construction of the App is stored in\n`source`.\n\nThe `verified` flag value indicates whether the app was used in the successful\nverification of a proof.","ref":"AppIdentity.App.html#t:t/0","title":"AppIdentity.App.t/0","type":"type"},{"doc":"A Plug that verifies App Identity proofs provided via one or more HTTP\nheaders.\n\nWhen multiple proof values are provided in the request, all must be\nsuccessfully verified. If any of the proof values cannot be verified,\nrequest processing halts with `403 Forbidden`. Should no proof headers be\nincluded, the request is considered invalid.\n\nAll of the above behaviours can be modified through\n[configuration](#module-configuration).","ref":"AppIdentity.Plug.html","title":"AppIdentity.Plug","type":"module"},{"doc":"The results of AppIdentity.Plug are stored in Plug.Conn private storage as\na map under the `:app_identity` key (this can be changed through the `name`\noption), keyed by the header group name. When using the `headers` option,\neach header is its own group. The `header_groups` option explicitly defines\nheaders that will be treated as belonging to the same group.\n\nHeader results only appear in the result map if they are present in the\nrequest. If AppIdentity.Plug is configured for `app1` and `app2` headers,\nbut there are only values in `app1`, the resulting output will not include\n`app2`.\n\n#","ref":"AppIdentity.Plug.html#module-checking-results","title":"Checking Results - AppIdentity.Plug","type":"module"},{"doc":"AppIdentity.Plug provides two main ways to partition processed results:\n`name` or header groups (either automatic grouping via `headers` or explicit\ngrouping via `header_groups`).\n\nMost applications that require result partitioning will use header groups,\nbecause there's only one pool of applications defined. In this case, use the\nfollowing configuration as a guide.\n\n```elixir\nplug AppIdentity.Plug, finder: &MyApp.Application.get/1,\n                       on_failure: :continue,\n                       header_groups: %{\n                          \"application\" => [\"application-identity\"],\n                          \"service\" => [\"service-identity\"]\n                       }\n```\n\nLater in request processsing, a controller or a route-specific Phoenix\npipeline could call a `require_application` function which pulls from\n`conn.private[:app_identity]` with the appropriate header group name for\nverification.\n\nIf there are _separate_ pools of applications defined, or there is a need to\nhave different `on_failure` conditions, then configure two\n`AppIdentity.Plug`s with different `name`s . The following example\nconfiguration would allow `application-identity` headers to fail without\nhalting (even if _omitted_), but a missing or incorrect `service-identity`\nheader would cause failures immediately.\n\n```elixir\nplug AppIdentity.Plug, finder: &MyApp.Application.get/1,\n                       on_failure: :continue,\n                       headers: [\"application-identity\"]\n\nplug AppIdentity.Plug, name: :service_app_identity,\n                       finder: {MyApp.ServiceApplication, :get},\n                       header: [\"service-identity\"]\n```\n\n> ##","ref":"AppIdentity.Plug.html#module-results-partitioning-header-groups-or-multiple-configurations","title":"Results Partitioning: Header Groups or Multiple Configurations? - AppIdentity.Plug","type":"module"},{"doc":">\n> If multiple AppIdentity.Plug configurations _are_ used, different `name`\n> values must be specified or the later plug will overwrite the results from\n> the earlier plug.","ref":"AppIdentity.Plug.html#module-multiple-plugs-warning-warning","title":"Multiple Plugs Warning {: .warning } - AppIdentity.Plug","type":"module"},{"doc":"AppIdentity.Plug requires configuration for app discovery and identity\nheaders and offers further configuration. Static configuration is strongly\nrecommended.\n\n#","ref":"AppIdentity.Plug.html#module-configuration","title":"Configuration - AppIdentity.Plug","type":"module"},{"doc":"So that AppIdentity.Plug can find apps used in identity validation, at least\none of `apps` or `finder` **must** be supplied. If both are present, the\n`apps` configuration is consulted before calling the `finder` function.\n\n- `apps`: A list of `t:AppIdentity.App.t/0` or `t:AppIdentity.App.input/0`\nvalues to be used for proof validation. Duplicate apps will be ignored.\n\n  ```elixir\n  plug AppIdentity.Plug, apps: [app1, app2], ...\n  ```\n\n- `finder`: A callback function conforming to `t:AppIdentity.App.finder/0`\n  that loads an `t:AppIdentity.App.input/0` from an external source given\n  a parsed proof. This may also be specified as a `{module, function}`\n  tuple.\n\n  ```elixir\n  plug AppIdentity.Plug, finder: &ApplicationModel.get/1\n  plug AppIdentity.Plug, finder: {ApplicationModel, :get}\n  ```\n\n  AppIdentity.Plug does not cache the results of the `finder` function. Any\n  caching should be implemented in your application.\n\n#","ref":"AppIdentity.Plug.html#module-app-discovery","title":"App Discovery - AppIdentity.Plug","type":"module"},{"doc":"AppIdentity.Plug does not have any default headers to search for app\nidentity validation, requiring one of `headers` or `header_groups` to be\nconfigured. If both are present, an exception will be raised during\nconfiguration.\n\n- `headers`: A list of valid HTTP header names, which will be normalized on\n  initialization.\n\n  ```elixir\n  plug AppIdentity.Plug, headers: [\"application-identity\"], ...\n  ```\n\n  The result output uses each header name as the key for the related proof\n  results. A configuration of `headers: [\"app1\", \"app2\"]` can produce\n  a result map like `%{\"app1\" => [...], \"app2\" => [...]}`.\n\n  Duplicate header names will result in an error. This option must be\n  omitted if `header_groups` is used.\n\n- `header_groups`: A map of header group names to valid HTTP header names.\n\n  When using `header_groups`, there is no guaranteed order for processing\n  groups, but the each headers within a group will be processed *in the\n  order provided*.\n\n  ```elixir\n  plug AppIdentity.Plug,\n    header_groups: %{\n                      \"application\" => [\"application\", \"my-application\"],\n                      \"service\" => [\"service\", \"my-service\"],\n                      },\n    ...\n  ```\n\n  The result output uses each header group name as the key for the related\n  proof results from any header in that group. A configuration of\n  `header_groups: %{\"app\" => [\"app1\", \"app2\"], \"svc\" => [\"svc1\"]}` can\n  produce a result map like `%{\"app\" => [...], \"svc\" => [...]}`.\n\n  Duplicate header names across any header groups will result in an error.\n  This option must be omitted if `headers` is used.\n\n> #### `headers` or `header_groups`? {: .info}\n>\n> The correct choice between `headers` and `header_groups` depends on your\n> application's requirements, but `headers` can be expressed as\n> `header_groups` for ease of changing later.\n>\n> That is, the following configurations are equivalent:\n>\n>   ```elixir\n>   plug AppIdentity.Plug, headers: [\"application-identity\"], ...\n>   plug AppIdentity.Plug,\n>     headers_groups: %{\"application-identity\" => [\"application-identity\"]}, ...\n>   ```\n>\n> If your requirements treat each header uniquely, `headers` is a useful\n> shorthand configuration.\n\n#","ref":"AppIdentity.Plug.html#module-identity-headers","title":"Identity Headers - AppIdentity.Plug","type":"module"},{"doc":"There are three configuration options that can be implemented as callbacks:\n`on_failure`, `on_success`, and `on_resolution`.\n\n- `on_failure`: The behaviour of the AppIdentity.Plug when proof validation\n  fails. If not provided, this defaults to `:forbidden`. When provided, it\n  must be one of the following values:\n\n  - `:forbidden`: Halt request processing and respond with a `403`\n    (forbidden) status. This is the same as `{:halt, :forbidden}`.\n\n    ```elixir\n    plug AppIdentity.Plug, on_failure: :forbidden\n    ```\n\n  - `{:halt, Plug.Conn.status()}`: Halt request processing and return the\n    specified status code. An empty body is emitted.\n\n    ```elixir\n    plug AppIdentity.Plug, on_failure: {:halt, :forbidden}\n    ```\n\n  - `{:halt, Plug.Conn.status(), Plug.Conn.body()}`: Halt request processing\n    and return the specified status code. The body value is included in the\n    response.\n\n    ```elixir\n    plug AppIdentity.Plug, on_failure: {:halt, :forbidden, [\"Evicted\"]}\n    ```\n\n  - `:continue`: Continue processing, ensuring that failure states are\n    recorded for the application to act on at a later point. This could be\n    used to implement a distinction between *validating* a proof and\n    *requiring* that the proof is valid.\n\n    ```elixir\n    plug AppIdentity.Plug, on_failure: :continue\n    ```\n\n  - A 1-arity callback function (or a `{module, function}` tuple) that\n    accepts a `Plug.Conn` and returns one of the above values. `on_failure`\n    callbacks **must not** modify the passed `conn` value.\n\n    ```elixir\n    plug AppIdentity.Plug, on_failure: {ApplicationModel, :resolve_proof_failure}\n    plug AppIdentity.Plug, on_failure: &ApplicationModel.resolve_proof_failure/1\n    ```\n\n- `on_success`: A 1-arity callback function (or a `{module, function}`\n  tuple) that accepts a `Plug.Conn` when proof validation succeeds. The\n  `on_success` callback **may** modify the passed `conn` value and must\n  return the modified `conn` value.\n\n- `on_resolution`: An 1-arity callback function (or a `{module, function}`\n  tuple) that accepts a `Plug.Conn` after proof validation completes,\n  regardless of sucesss or failure. If present, this will be run as the last\n  step (after `on_failure` and `on_success`). Because `on_failure` may halt\n  pipeline processing, it may be necessary to check `conn.halted`. The\n  `on_resolution` callback **may** modify the passed `conn` value and must\n  return the modified `conn` value.\n\nThe `on_success` and `on_resolution` callbacks are optional.\n\n#","ref":"AppIdentity.Plug.html#module-callbacks","title":"Callbacks - AppIdentity.Plug","type":"module"},{"doc":"- `name`: An atom which will be used to store the `AppIdentity.Plug` results\n  in Plug.Conn private storage. If not provided, defaults to\n  `:app_identity`. Required if `AppIdentity.Plug` will be specified more\n  than once as results are not merged.\n\n  ```elixir\n  plug AppIdentity.Plug, name: :service_app, ...\n  ```\n\n- `disallowed`: A list of algorithm versions that are not allowed when\n  processing received identity proofs. See `t:AppIdentity.disallowed/0`.\n\n  ```elixir\n  plug AppIdentity.Plug, disallowed: [1], ...\n  ```","ref":"AppIdentity.Plug.html#module-optional-configuration","title":"Optional Configuration - AppIdentity.Plug","type":"module"},{"doc":"When telemetry is enabled, this plug will emit `[:app_identity, :plug,\n:start]` and `[:app_identity, :plug, :stop]` events.","ref":"AppIdentity.Plug.html#module-telemetry","title":"Telemetry - AppIdentity.Plug","type":"module"},{"doc":"AppIdentity.Plug configuration struct builder and telemetry transformer.","ref":"AppIdentity.Plug.Config.html","title":"AppIdentity.Plug.Config","type":"module"},{"doc":"Create and validate the configuration struct from input parameters.","ref":"AppIdentity.Plug.Config.html#new!/1","title":"AppIdentity.Plug.Config.new!/1","type":"function"},{"doc":"Returns the plug configuration as data for Telemetry context consumption.","ref":"AppIdentity.Plug.Config.html#telemetry_context/1","title":"AppIdentity.Plug.Config.telemetry_context/1","type":"function"},{"doc":"The internal representation of a callback function.","ref":"AppIdentity.Plug.Config.html#t:callback/1","title":"AppIdentity.Plug.Config.callback/1","type":"type"},{"doc":"A callback for either `on_success` or `on_resolution` which accepts and\noptionally transforms the request `conn`.","ref":"AppIdentity.Plug.Config.html#t:conn_callback/0","title":"AppIdentity.Plug.Config.conn_callback/0","type":"type"},{"doc":"","ref":"AppIdentity.Plug.Config.html#t:on_failure/0","title":"AppIdentity.Plug.Config.on_failure/0","type":"type"},{"doc":"A callback for `on_failure` configuration. It must return a normal\n`on_failure` value.","ref":"AppIdentity.Plug.Config.html#t:on_failure_callback/0","title":"AppIdentity.Plug.Config.on_failure_callback/0","type":"type"},{"doc":"AppIdentity.Plug configuration options prior to validation.","ref":"AppIdentity.Plug.Config.html#t:param/0","title":"AppIdentity.Plug.Config.param/0","type":"type"},{"doc":"Normalized Configuration for AppIdentity.Plug.","ref":"AppIdentity.Plug.Config.html#t:t/0","title":"AppIdentity.Plug.Config.t/0","type":"type"},{"doc":"A struct describing a computed or parsed App Identity proof.","ref":"AppIdentity.Proof.html","title":"AppIdentity.Proof","type":"module"},{"doc":"Build an `AppIdentity.Proof` struct from an `AppIdentity.App` struct and\na nonce.\n\nThis method returns the either `{:ok, proof}` or `{:error, reason}`","ref":"AppIdentity.Proof.html#build/3","title":"AppIdentity.Proof.build/3","type":"function"},{"doc":"iex> AppIdentity.Proof.build(\n    ...>   AppIdentity.App.new!(%{\n    ...>     id: \"867-5309\",\n    ...>     secret: \"Something unguessable\",\n    ...>     version: 1\n    ...>   }),\n    ...>   \"a nonce\"\n    ...> )\n    {:ok, %AppIdentity.Proof{\n      id: \"867-5309\",\n      nonce: \"a nonce\",\n      version: 1,\n      padlock: \"57384B64BF69703296DF9E4C93DC83CF4B96EAF20298006897DFBC2A63904219\"\n    }}\n\n    iex> AppIdentity.Proof.build(\n    ...>   AppIdentity.App.new!(%{\n    ...>     id: \"123-456-789\",\n    ...>     secret: fn -> \"Something unguessable\" end,\n    ...>     version: 2\n    ...>   }),\n    ...>   \"\",\n    ...>   version: 3\n    ...> )\n    {:error, \"nonce must not be an empty string\"}","ref":"AppIdentity.Proof.html#build/3-examples","title":"Examples: - AppIdentity.Proof.build/3","type":"function"},{"doc":"","ref":"AppIdentity.Proof.html#from_string/1","title":"AppIdentity.Proof.from_string/1","type":"function"},{"doc":"","ref":"AppIdentity.Proof.html#to_string/1","title":"AppIdentity.Proof.to_string/1","type":"function"},{"doc":"","ref":"AppIdentity.Proof.html#verify/3","title":"AppIdentity.Proof.verify/3","type":"function"},{"doc":"The components of a parsed AppIdentity proof.","ref":"AppIdentity.Proof.html#t:t/0","title":"AppIdentity.Proof.t/0","type":"type"},{"doc":"If [telemetry](https://hexdocs.pm/telemetry) is a dependency in your\napplication, and the telemetry is not explicitly disabled, telemetry events\nwill be emitted for `AppIdentity.generate_proof/2`,\n`AppIdentity.verify_proof/3`, and `AppIdentity.Plug`. See\n`AppIdentity.Telemetry` for more information.","ref":"AppIdentity.Telemetry.html","title":"AppIdentity.Telemetry","type":"module"},{"doc":"All of `AppIdentity`'s telemetry events are spans, consisting of `:start` and\n`:stop` events. These are always in the form `[:app_identity,\n ,  ]`.\n\nThe events are:\n\n- `[:app_identity, :generate_proof, :start]` emitted when\n  `AppIdentity.generate_proof/2` or `AppIdentity.generate_proof!/2` is called.\n- `[:app_identity, :generate_proof, :stop]` emitted when\n  `AppIdentity.generate_proof/2` or `AppIdentity.generate_proof!/2` finishes,\n  either on success or failure. Telemetry is generated *prior* to the raising\n  of any exceptions for `AppIdentity.generate_proof/2`.\n- `[:app_identity, :plug, :start]` emitted when `AppIdentity.Plug` is\n  called. If `AppIdentity.Plug` is called in-line, exceptions may be thrown\n  if there are configuration issues and telemetry will not be emitted.\n- `[:app_identity, :plug, :stop]` emitted when the response is sent after\n  `AppIdentity.Plug` completes.\n- `[:app_identity, :verify_proof, :start]` emitted when\n  `AppIdentity.verify_proof/3` or `AppIdentity.verify_proof!/3` is called.\n- `[:app_identity, :verify_proof, :stop]` emitted when\n  `AppIdentity.verify_proof/3` or `AppIdentity.verify_proof!/3` finishes,\n  either on success or failure. Telemetry is verified *prior* to the raising\n  of any exceptions for `AppIdentity.verify_proof/3`.\n\n#","ref":"AppIdentity.Telemetry.html#module-telemetry-events","title":"Telemetry Events - AppIdentity.Telemetry","type":"module"},{"doc":"All `AppIdentity` telemetry events measure one of two things:\n\n- `:start` events always measure `monotonic_time` (`:erlang.monotonic_time/0`)\n  and `system_time` (`:erlang.system_time/0`).\n\n- `:stop` events always measure `monotonic_time` (`:erlang.monotonic_time/0`)\n  and `duration` (the difference between the start time and stop time).\n\n#","ref":"AppIdentity.Telemetry.html#module-measurements","title":"Measurements - AppIdentity.Telemetry","type":"module"},{"doc":"All `AppIdentity` telemetry includes a `telemetry_span_context` key which is\na `t::erlang.reference/0`. In this version, this value will be generated with\nthe `:start` event and reused for the `:stop` event.\n\nWhen a measurement type is given as `t:telemetry_app`, this will be one of the\nfollowing values:\n\n#### `[:app_identity, :generate_proof, :start]` Metadata\n\n- `app`: `t:telemetry_app/0`\n- `options`: a list of `t:AppIdentity.option/0`\n\n#### `[:app_identity, :generate_proof, :stop]` Metadata\n\n- `app`: `t:telemetry_app/0`\n- `options`: a list of `t:AppIdentity.option/0`\n- One of:\n  - `proof`: The generated proof string\n  - `error`: A descriptive error\n\n#### `[:app_identity, :plug, :start]` Metadata\n\n- `conn`: `t:Plug.Conn.t/0`\n- `options`: A map that may have the following keys (derived from\n  `t:AppIdentity.Plug.Config.param/0`):\n  - `apps`: a list of `t:telemetry_app/0` values\n  - `finder`: if present, the string `\"function (anonymous)\"` for an anonymous\n    finder function or `\"function (module.function/1)\"` for a finder function\n    specified by name.\n  - `headers`: a list of identity header names\n  - `header_groups`: a map of header groups to lists of header names\n  - `on_failure`: the atom value of `t:AppIdentity.Plug.Config.on_failure/0`\n    or if the `on_failure` configuration is\n    `t:AppIdentity.Plug.Config.on_failure_callback/0`, it will be either the\n    string `\"function (anonymous)\"` for an anonymous callback or `\"function\n    (module.function/1)\"` for a named callback.\n  - `on_success`: if present, either the string `\"function (anonymous)\"` for\n    an anonymous callback or `\"function (module.function/1)\"` for a named\n    callback.\n  - `on_resolution`: if present, either the string `\"function (anonymous)\"`\n    for an anonymous callback or `\"function (module.function/1)\"` for a named\n    callback.\n  - `disallowed`: a `t:AppIdentity.disallowed/0` value.\n\n#### `[:app_identity, :plug, :stop]` Metadata\n\n- `conn`: `t:Plug.Conn.t/0`, updated after processing\n- `options`: A map with the following keys (derived from\n  `t:AppIdentity.Plug.Config.param/0`):\n  - `apps`: a list of `t:telemetry_app/0` values\n  - `finder`: if present, the string `\"function (anonymous)\"` for an anonymous\n    finder function or `\"function (module.function/1)\"` for a finder function\n    specified by name.\n  - `headers`: a list of identity header names\n  - `header_groups`: a map of header groups to lists of header names\n  - `on_failure`: the atom value of `t:AppIdentity.Plug.Config.on_failure/0`\n    or if the `on_failure` configuration is\n    `t:AppIdentity.Plug.Config.on_failure_callback/0`, it will be either the\n    string `\"function (anonymous)\"` for an anonymous callback or `\"function\n    (module.function/1)\"` for a named callback.\n  - `on_success`: if present, either the string `\"function (anonymous)\"` for\n    an anonymous callback or `\"function (module.function/1)\"` for a named\n    callback.\n  - `on_resolution`: if present, either the string `\"function (anonymous)\"`\n    for an anonymous callback or `\"function (module.function/1)\"` for a named\n    callback.\n  - `disallowed`: a `t:AppIdentity.disallowed/0` value.\n\n#### `[:app_identity, :verify_proof, :start]` Metadata\n\n- `app`: `t:telemetry_app/0`\n- `options`: a list of `t:AppIdentity.option/0`\n- One of:\n  - `candidate`: The candidate proof string\n  - `proof`: The parsed proof\n\n#### `[:app_identity, :verify_proof, :stop]` Metadata\n\n- `app`: `t:telemetry_app/0`. In the case of an error, this will be the same\n  `app` value as was reported in the `:start` event. If not, this value will\n  be the post-verification version of the `app`.\n- `options`: a list of `t:AppIdentity.option/0`\n- One of:\n  - `candidate`: The candidate proof string\n  - `proof`: The parsed proof\n- In case of an error:\n  - `error`: A descriptive error","ref":"AppIdentity.Telemetry.html#module-metadata","title":"Metadata - AppIdentity.Telemetry","type":"module"},{"doc":"Telemetry may be disabled by setting this for your configuration:\n\n    config :app_identity, AppIdentity.Telemetry, enabled: false\n\nRecompile `app_identity` if this setting is changed:\n\n```console\n$ mix deps.compile --force app_identity\n```","ref":"AppIdentity.Telemetry.html#module-disabling-telemetry","title":"Disabling Telemetry - AppIdentity.Telemetry","type":"module"},{"doc":"A telemetry-safe version of an input or verified app.\n\nMay be one of the following values:\n\n- `nil`: the app is not found or does not verify correctly.\n- `\"loader\"`: the app provided is a `t:AppIdentity.App.loader/0` function.\n- `\"finder\"`: the app provided is a `t:AppIdentity.App.finder/0` function.\n- Otherwise, a map is returned with required `:id` and `:version` keys and\n  optional `:config` and `:verified` keys.","ref":"AppIdentity.Telemetry.html#t:telemetry_app/0","title":"AppIdentity.Telemetry.telemetry_app/0","type":"opaque"},{"doc":"A Tesla middleware that generates an app identity proof header for\na request.\n\nThe `options` provided has the following parameters:\n\n- `app`: (required) A value that can be passed to AppIdentity.App.new/1.\n\n- `disallowed`: A list of algorithm versions that are not allowed when\n  processing received identity proofs. See `t:AppIdentity.disallowed/0`.\n\n- `header`: (required) The valid HTTP header to use for sending the app\n  identity proof.\n\n- `on_failure`: (optional) The action to take when an app identity proof\n  cannot be generated for any reason *except* configuration errors. May be\n  one of the following values:\n\n  - `:fail`: Fails the request with an error tuple. This is the default if\n    `on_failure` is not specified.\n\n  - `:pass`: Sets the header to the empty value returned. The request will\n    probably fail on the receiving server side.\n\n  - `:skip`: Does not add the header, as if the request were not made using\n    an application.\n\n`on_failure` may also be provided an arity 3 function or `{module,\nfunction}` tuple that expects three parameters:\n\n- `env`: The Tesla middleware `env` value;\n- `app`: The identity app value provided to the middleware; and\n- `header`: The header name provided to the middleware.\n\nThe function may return either the `env`, `:fail`, `:skip`, or `:pass`. Any\nother value will be treated as `:fail`.","ref":"AppIdentity.TeslaMiddleware.html","title":"AppIdentity.TeslaMiddleware","type":"module"},{"doc":"","ref":"AppIdentity.TeslaMiddleware.html#t:on_failure/0","title":"AppIdentity.TeslaMiddleware.on_failure/0","type":"type"},{"doc":"","ref":"AppIdentity.TeslaMiddleware.html#t:on_failure_callback/0","title":"AppIdentity.TeslaMiddleware.on_failure_callback/0","type":"type"},{"doc":"","ref":"AppIdentity.TeslaMiddleware.html#t:option/0","title":"AppIdentity.TeslaMiddleware.option/0","type":"type"},{"doc":"Generate or run an app identity test suite","ref":"Mix.Tasks.AppIdentity.html","title":"mix app_identity","type":"task"},{"doc":"mix app_identity [options] [command]","ref":"Mix.Tasks.AppIdentity.html#module-usage","title":"Usage - mix app_identity","type":"task"},{"doc":"-V, --version    Display the version number\n    -h, --help       Display help","ref":"Mix.Tasks.AppIdentity.html#module-options","title":"Options - mix app_identity","type":"task"},{"doc":"generate          Generates an integration test suite\n    run               Runs one or more integration suites\n    help              Display help for command","ref":"Mix.Tasks.AppIdentity.html#module-commands","title":"Commands - mix app_identity","type":"task"},{"doc":"","ref":"Mix.Tasks.AppIdentity.html#run/1","title":"Mix.Tasks.AppIdentity.run/1","type":"function"},{"doc":"An exception that will be raised by `AppIdentity.generate_proof!/2`,\n`AppIdentity.parse_proof!/1`, or `AppIdentity.verify_proof!/3` when those\nfunctions fail. The error message reported here is *always* generic.\n\nThis can also be raised by `AppIdentity.Plug` on configuration failure.","ref":"AppIdentity.AppIdentityError.html","title":"AppIdentity.AppIdentityError","type":"exception"},{"doc":"# AppIdentity for Elixir\n\n- code :: https://github.com/KineticCafe/app-identity/tree/main/elixir\n- issues :: https://github.com/KineticCafe/app-identity/issues","ref":"readme.html","title":"AppIdentity for Elixir","type":"extras"},{"doc":"AppIdentity is an Elixir implementation of the Kinetic Commerce application\nidentity proof algorithm as described in its [spec][].","ref":"readme.html#description","title":"Description - AppIdentity for Elixir","type":"extras"},{"doc":"```elixir\napp = %{id: id, secret: secret, version: 2}\nproof = AppIdentity.generate_proof(app)\nAppIdentity.verify_proof(proof, app)\n```\n\nThere is a Plug available for authenticating applications, `AppIdentity.Plug`.\n\n```elixir\nplug AppIdentity.Plug,\n  headers: [\"app-identity-proof\"],\n  finder: fn %{id: id} = _proof -> IdentityApplications.get(id) end\n\n```\n\nThere is a Tesla Middleware for providing proof generation for clients.\n\n```elixir\ndef client(app) do\n  middleware = [\n    {AppIdentity.TeslaMiddleware, identity_app: app, header: \"app-identity-proof\"}\n  ]\nend\n```","ref":"readme.html#synopsis","title":"Synopsis - AppIdentity for Elixir","type":"extras"},{"doc":"The package can be installed by adding `app_identity` to your list of\ndependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:app_identity, \"~> 1.3\"}\n  ]\nend\n```\n\nIf you need to use a pre-release version, the dependency structure is slightly\ndifferent:\n\n```elixir\ndef deps do\n  [\n    {:app_identity, github: \"KineticCafe/app_identity\", sparse: \"elixir\"}\n  ]\nend\n```\n\nIf on Elixir 1.13 or later, you can use `:subdir` instead:\n\n```elixir\ndef deps do\n  [\n    {:app_identity, github: \"KineticCafe/app_identity\", subdir: \"elixir\"}\n  ]\nend\n```\n\nOptional features are present when [Plug][] and/or [Tesla][] are part of your\napplication.\n\nDocumentation can found at [HexDocs][docs] or generated with [ex_doc][].","ref":"readme.html#installation","title":"Installation - AppIdentity for Elixir","type":"extras"},{"doc":"`AppIdentity` uses a [Semantic Versioning][] scheme with one significant change:\n\n- When PATCH is zero (`0`), it will be omitted from version references.\n\nAdditionally, the major version will generally be reserved for specification\nrevisions.","ref":"readme.html#semantic-versioning","title":"Semantic Versioning - AppIdentity for Elixir","type":"extras"},{"doc":"AppIdentity for Elixir [welcomes contributions](Contributing.md). This project,\nlike all Kinetic Commerce [open source projects][], is under the Kinetic\nCommerce Open Source [Code of Conduct][kccoc].\n\nAppIdentity for Elixir is licensed under the Apache Licence, version 2.0 and\nrequires certification of a Developer Certificate of Origin. See\n[Licence.md](Licence.md) for more details.\n\n[docs]: https://hexdocs.pm/app_identity\n[ex_doc]: https://github.com/elixir-lang/ex_doc\n[kccoc]: https://github.com/KineticCafe/code-of-conduct\n[open source projects]: https://github.com/KineticCafe\n[plug]: https://hex.pm/packages/plug\n[semantic versioning]: http://semver.org/\n[spec]: https://github.com/KineticCafe/app-identity/blob/main/spec/README.md\n[tesla]: https://hex.pm/packages/tesla","ref":"readme.html#contributing","title":"Contributing - AppIdentity for Elixir","type":"extras"},{"doc":"# Contributing\n\nWe value contributions to AppIdentity for Elixir—bug reports, discussions,\nfeature requests, and code contributions. New features should be proposed and\n[discussed][] prior to implementation, and release of any new feature may be\ndelayed until implemented in the three reference implementations.\n\nBefore contributing patches, please read the [Licence.md](Licence.md).\n\nApp Identity is governed under the Kinetic Commerce Open Source [Code of\nConduct][].","ref":"contributing.html","title":"Contributing","type":"extras"},{"doc":"Our usual code contribution guidelines apply:\n\n- Code changes _will not_ be accepted without tests.\n- We use `mix format` and [Credo][credo] for consistent coding style and\n  formatting.\n- Use a thoughtfully-named topic branch that contains your change. Rebase your\n  commits into logical chunks as necessary.\n- Use [quality commit messages][].\n- The version number must not be changed except as part of the release process.\n- Submit a pull request with your changes.\n- New or changed behaviours require new or updated documentation.\n- New dependencies are discouraged, unless they are `optional` (as we have done\n  for Plug and Tesla).\n\nThere are code quality checks performed in GitHub Actions that must pass for any\npull request to be accepted.\n\n[code of conduct]: https://github.com/KineticCafe/code-of-conduct\n[credo]: https://github.com/rrrene/credo\n[discussed]: https://github.com/KineticCafe/app_identity/discussions\n[hoe]: https://github.com/seattlerb/hoe\n[quality commit messages]: http://tbaggery.com/2008/04/19/a-note-about-git-commit-messages.html","ref":"contributing.html#code-guidelines","title":"Code Guidelines - Contributing","type":"extras"},{"doc":"# App Identity for Elixir Changelog","ref":"Changelog.md.html","title":"Changelog","type":"extras"},{"doc":"- Packages released to hex.pm by default include the `priv/` directory, but\n  nothing is done to ensure that those packages include anything other than the\n  literal files. The `priv/` directory contained two symbolic links\n  (`optional.json` and `required.json`) which were symlinks to the project\n  integration suite generators, only used in integration testing.","ref":"Changelog.md.html#1-3-1-2023-07-20","title":"1.3.1 / 2023-07-20 - Changelog","type":"extras"},{"doc":"- Rename all spec uses of `String.t()` to `binary()` as\n\n- Extensive reorganization of the `AppIdentity.Plug` documentation to improve\n  the readability of the configuration.\n\n- Refactored configuration into `AppIdentity.Plug.Config` from the plug itself.\n  This was done in part to resolve a Dialyzer issue.\n\n- Add `on_resolution` and `on_success` callbacks in `AppIdentity.Plug.Config` to\n  better support various workflows (such as adding proof validation results to\n  `Logger.metadata/1`).\n\n- Extended the `AppIdentity.Plug.Config.finder` callback to accept a tuple\n  `{module, function}`.\n\n- Improved `AppIdentity.Plug.Config` telemetry context formatting to include the\n  plug `name`.","ref":"Changelog.md.html#1-3-0-2023-07-19","title":"1.3.0 / 2023-07-19 - Changelog","type":"extras"},{"doc":"- Add support for header groups in `AppIdentity.Plug` to better handle fallback\n  headers. Kinetic’s original Elixir implementation always verified only the\n  _first_ value from a _list_ of headers, like so:\n\n  ```elixir\n  with []   {:ok, value}\n  end\n  ```\n\n  AppIdentity.Plug always processes all values of a header and puts the result\n  in a map with the header name as the key, it meant that each header result\n  would need to be checked individually. Instead, the `header_groups` option\n  collects _related_ headers into a single result key:\n\n  ```elixir\n  plug AppIdentity.Plug, header_groups: %{\n    \"app\" => [\"header-1\", \"header-2\", \"header-3\"]\n  }, ...\n  ```\n\n- Add support for alternate names so that `AppIdentity.Plug` can be specified\n  multiple times in a pipeline and will store its data separately.","ref":"Changelog.md.html#1-2-0-2023-07-07","title":"1.2.0 / 2023-07-07 - Changelog","type":"extras"},{"doc":"- Add optional Telemetry support. If `:telemetry` is in your application's\n  dependencies, and Telemetry support is not explicitly disabled, events will be\n  emitted for `AppIdentity.generate_proof/2`, `AppIdentity.verify_proof/3`, and\n  `AppIdentity.Plug`.\n\n  Disable by adding this line to your application's configuration:'\n\n  ```elixir\n  config :app_identity, AppIdentity.Telemetry, enabled: false\n  ```\n\n- Fixed various issues on Elixir 1.10.","ref":"Changelog.md.html#1-1-0-2023-03-28","title":"1.1.0 / 2023-03-28 - Changelog","type":"extras"},{"doc":"- Initial release.","ref":"Changelog.md.html#1-0-0-2022-09-07","title":"1.0.0 / 2022-09-07 - Changelog","type":"extras"},{"doc":"# App Identity Specification\n\nApp Identity provides a fast, lightweight, cryptographically secure app\nauthentication mechanism as an improvement over just using API keys or app IDs.\nIt does this by computing a proof with an application identifier, a nonce, an\napplication secret key, and a hashing algorithm. The secret key is embedded in\nclient applications and stored securely on the server, so it is never passed\nover the wire.","ref":"spec.html","title":"App Identity Specification","type":"extras"},{"doc":"The key words \"MUST\", \"MUST NOT\", \"REQUIRED\", \"SHALL\", \"SHALL NOT\", \"SHOULD\",\n\"SHOULD NOT\", \"RECOMMENDED\", \"MAY\", and \"OPTIONAL\" in this document are to be\ninterpreted as described in [RFC2199][rfc2119].\n\nOnly the text of this specification is _normative_. Code examples in this\ndocument are _informative_ and may have bugs.","ref":"spec.html#terminology-notes","title":"Terminology Notes - App Identity Specification","type":"extras"},{"doc":"This specification is versioned with a modified [Semantic Versioning][] scheme.\nThe major version of the specification will always be the highest algorithms\nversion defined. Minor versions may adjust the text of the specification.\n\nThe current specification is `4.1`, as four algorithm versions have been\ndefined.","ref":"spec.html#version-and-versioning","title":"Version and Versioning - App Identity Specification","type":"extras"},{"doc":"For the purposes of this specification, an `application` requires the following\nattributes:\n\n- `id`: The unique identifier of the application. It is recommended that this\n  value is a `UUID`. When using an integer identifier, it is recommended that\n  this value be extended, such as that provided by Rails [Global ID][global id].\n  Such representations are _also_ recommended if the ID is a compound value.\n  Non-string identifiers must be converted to string values.\n\n- `secret`: The random value used as the secret key. This value _must_ be used\n  as presented (if presented as a base-64 value, the secret _is_ the base-64\n  value, not a decoded version of it).\n\n  Application secrets should be prefixed with a fixed value, such as `appid_`.\n  This improves the ability of security tools (such as [gitleaks][],\n  [GitGuardian][], or [GitHub secret scanning][]) to detect that an application\n  secret key has been leaked\n\n  Where possible, implementations **should** use memory-safe storage for the\n  `secret` value and **must** prevent accidental exposure of the secret in logs\n  through normal object introspection.\n\n  > All reference implementations use no-argument closures to store the secret\n  > in memory, which prevents accidental logging of the secret. In addition,\n  > custom inspect functions have been implemented which hides the secret value\n  > by default.\n\n- `version`: The minimum algorithm version supported by the application, an\n  integer value. The reference implementations of App Identity do not restrict\n  the `version` for backwards compatibility purposes, but new services\n  **should** consider version 1 applications deprecated.\n\n- `config`: A configuration object which affects proof verification. Only one\n  key for this object is defined when `version` 2 or higher. The `config` only\n  affects proof verification.\n\n  - `fuzz`: The fuzziness of time stamp comparison, in seconds, for version `2`\n    or higher algorithms. If not present, defaults to `600` seconds, or ±600\n    seconds (±10 minutes). Depending on the nature of the app being verified and\n    the expected network conditions, a shorter time period than 600 seconds is\n    recommended.\n\n#","ref":"spec.html#application","title":"Application - App Identity Specification","type":"extras"},{"doc":"The following fields are recommended for use on by proof verifiers.\n\n- `code`: A unique text identifier for the application. Not used in the proof\n  algorithm, but used to manage the application on the servers.\n\n- `name`: A displayable name for the application. Not used in the proof\n  algorithm, but used for human consumption.\n\n- `type`: The type of the application. Because the server deals with many types\n  of applications, this value may be used to _limit_ access to APIs to only type\n  of applications.","ref":"spec.html#suggested-extra-fields","title":"Suggested Extra Fields - App Identity Specification","type":"extras"},{"doc":"The algorithm version controls both the shape of the nonce and the specific\nalgorithm chosen for hashing. Versions are _strictly upgradeable_. That is,\na version 1 app can verify version 1, 2, 3, or 4 proofs. However, a version\n2 app will _never_ validate a version 1 proof.\n\n \n   \n     \n       App Identity Version \n       Nonce \n       Digest Algorithm \n       Can Verify Proof from Version \n     \n      1  2  3  4  \n   \n   \n     \n       1 \n       random \n       SHA2-256 \n       Yes \n       Yes \n       Yes \n       Yes \n     \n     \n       2 \n       timestamp ± fuzz \n       SHA2-256 \n       - \n       Yes \n       Yes \n       Yes \n     \n     \n       3 \n       timestamp ± fuzz \n       SHA2-384 \n       - \n       - \n       Yes \n       Yes \n     \n     \n       4 \n       timestamp ± fuzz \n       SHA2-512 \n       - \n       - \n       - \n       Yes","ref":"spec.html#algorithm-versions","title":"Algorithm Versions - App Identity Specification","type":"extras"},{"doc":"The client identity proof is a short signed value, composed from the _id_,\na _nonce_, and an intermediary _padlock_ generated using the _application\nsecret_. The application id and secret will be provided securely for\ncompile-time inclusion; all care should be taken to ensure that the secret is\nnot easily extractable from the application or shared in the clear.\n\nThe generation of a proof looks like this:\n\n    padlock = Padlock(Version, Identity, Nonce, Secret)\n    proof = Proof(Version, Identity, Nonce, padlock)\n\nThe verification of a proof looks like this:\n\n    (Version, Identity, Nonce, padlock) = Parse(proof)\n    app = FindApplication(Identity)\n    padlockʹ = Padlock(Version, app.Identity, Nonce, app.Secret)\n    proofʹ = Proof(Version, Identity, Nonce, padlockʹ)\n\n    ConstantTimeEqualityComparison(proof, proofʹ)\n\n#","ref":"spec.html#identity-proof","title":"Identity Proof - App Identity Specification","type":"extras"},{"doc":"Depending on the version of the application algorithm, the _nonce_ may contain\nany byte sequences _except_ ASCII colon (`:`), but it is recommended that the\nvalue be UTF-8 safe.\n\n##","ref":"spec.html#nonce","title":"Nonce - App Identity Specification","type":"extras"},{"doc":"Version 1 nonces should be cryptographically secure and non-sequential, but\nsufficiently fine-grained timestamps (those including microseconds, as\n`yyyymmddHHMMSS.sss`) _may_ be used. Version 1 proofs verify that the nonce is\nat least one byte long and do not contain a colon (`:`).\n\n**Ruby**:\n\n```ruby\nrequire 'securerandom'\nnonce1 = SecureRandom.urlsafe_base64(32)\nnonce2 = SecureRandom.hex(32)\n```\n\n**Elixir**:\n\n```elixir\n# Elixir\nBase.url_encode64(:crypto.strong_rand_bytes(32), padding: true)\nBase.encode16(:crypto.strong_rand_bytes(32))\n```\n\n**Typescript (Node)**:\n\n```typescript\nimport { randomBytes } from 'crypto'\nimport base64url from 'base64-url' // https://www.npmjs.com/package/base64-url\n\nbase64url.encode(randomBytes(32).toString())\n```\n\n**Swift**:\n\n```swift\nfunc secure_random_base64_bytes(count: Int32 = 16) -> String? {\n  var data = Data(count: count)\n  let result = data.withUnsafeMutableBytes {\n    SecRandomCopyBytes(kSecRandomDefaults, data.count, $0)\n  }\n  if result == errSecSuccess {\n    return data.base64EncodedString()\n  } else {\n    return nil\n  }\n}\n\nsecure_random_base64_bytes(32)\n```\n\n##","ref":"spec.html#random-nonces","title":"Random Nonces - App Identity Specification","type":"extras"},{"doc":"Version 2, 3, and 4 nonces **must** be a UTC timestamp formatted using ISO 8601\nbasic formatting. The timestamp _should_ be generated on a clock synced with NTP\nand _should_ be verified using a clock synced with NTP.\n\nFor the purposes of this document, ISO 8601 basic formatting is this [ABNF][]\ndefinition adapted from [RFC3339][]:\n\n```abnf\ndate-fullyear = 4DIGIT\ndate-month    = 2DIGIT  ; 01-12\ndate-mday     = 2DIGIT  ; 01-28, 01-29, 01-30, 01-31 based on\n                        ; month/year\ntime-hour     = 2DIGIT  ; 00-23\ntime-minute   = 2DIGIT  ; 00-59\ntime-second   = 2DIGIT  ; 00-58, 00-59, 00-60 based on leap second\n                        ; rules\ntime-secfrac  = \".\" 1\\*DIGIT\ntime-offset   = \"Z\"\n\npartial-time  = time-hour time-minute time-second [time-secfrac]\nfull-date     = date-fullyear date-month date-mday\nfull-time     = partial-time time-offset\n\ndate-time     = full-date \"T\" full-time\n```\n\nThe timestamp must be an ASCII 7-bit or UTF-8 string using only ASCII\ncharacters, and the special characters `T` and `Z` **must** be specified\nuppercase.\n\nThis format differs from [RFC3339][] [§5.6][§5.6] timestamp format in the\nfollowing ways:\n\n1. It **must** be UTC and the timezone character must be `Z`. No other timezone\n   specifier is permitted, and it must not be omitted.\n2. It **must** only have the characters `[.0-9TZ]`. It **may** have the point\n   character (`.`) only preceding the _optional_ fractional seconds digits.\n\nTherefore, a timestamp of `2020-02-25T23:20:03.321423-04:00` must be presented\nas `20200225T192003.321423Z`.\n\nThe C-style `strftime` pattern for this format is `'%Y%m%dT%H%M%S.%6NZ'`, and\nthe PostgreSQL `TO_CHAR` pattern is `'YYYYMMDD\"T\"HH24MISS.FF6Z'`.\n\n**Ruby**:\n\n```ruby\nrequire 'time'\nTime.now.utc.strftime('%Y%m%dT%H%M%S.%6NZ')\n```\n\n**Elixir**:\n\n```elixir\ncase DateTime.now(\"Etc/UTC\") do\n  {:ok, stamp} ->\n    {:ok, DateTime.to_iso8601(stamp, :basic)}\n\n  {:error, reason} ->\n    {:error, String.replace(Kernel.to_string(reason), \"_\", \" \")}\nend\n```\n\n**Typescript (Node)**:\n\n```typescript\nnew Date().toISOString().replace(/[-:]/g, '')\n```\n\n#","ref":"spec.html#timestamp-nonces","title":"Timestamp Nonces - App Identity Specification","type":"extras"},{"doc":"To compute the padlock value, concatenate the application id, the nonce, and the\napplication secret using colons, then calculate the digest of the value and\nconvert to a base 16 string representation. As noted previously, the digest\nalgorithm used varies based on the application version.\n\n**Ruby**:\n\n```ruby\nrequire 'digest/sha2'\nDigest::SHA256.hexdigest([version, id, nonce, secret].join(':')).upcase\n```\n\n**Elixir**:\n\n```elixir\n[id, nonce, secret]\n|> Enum.join(\":\")\n|> then(&:crypto.hash(:sha256, &1))\n|> Base.encode16(case: :upper)\n```\n\n**Typescript (Node)**:\n\n```typescript\nimport { createHash } from 'crypto'\nconst hash = createHash('sha384')\nhash.update(raw, 'utf-8')\nhash.digest('hex').toUpperCase()\n```\n\n**Swift**:\n\n```swift\n// Swift\nextension Data {\n  func hexString() -> String {\n    return self.map {\n        Int($0).hexString()\n    }.joined()\n  }\n\n  func SHA256() -> Data {\n    var result = Data(count: Int(CC_SHA256_DIGEST_LENGTH))\n    _ = result.withUnsafeMutableBytes {\n      resultPtr in self.withUnsafeBytes {\n        CC_SHA256($0, CC_LONG(count), resultPtr)\n      }\n    }\n    return result\n  }\n}\n\nextension String {\n  func hexString() -> String {\n    return self.data(using: .utf8)!.hexString()\n  }\n\n  func SHA256() -> String {\n    return self.data(using: .utf8)!.SHA256().hexString()\n  }\n}\n\nlet value = \"\\(id):\\(nonce):\\(secret)\"\nlet padlock = value.SHA256()\n```\n\n**Java**:\n\n```java\n// Java (Android)\nString value = id + \":\" + nonce + \":\" + secret;\nMessageDigest digest = MessageDigest.getInstance(\"SHA-256\");\nbyte[] hash = digest.digest(msg.getBytes(StandardCharsets.UTF_8));\nStringBuffer padlock = new StringBuffer();\n\nfor (int i = 0; i < hash.length; i++) {\n  String hex = Integer.toHexString(0xFF & hash[i]);\n  if (hex.length() == 1) {\n    padlock.append('0');\n  }\n\n  padlock.append(hex);\n}\n```\n\nValidation of the padlock will convert this digest to uppercase, so the values\n`c0ffee` and `C0FFEE` are identical. It is recommended that padlocks be passed\nas uppercase hex values.\n\n#","ref":"spec.html#padlock-calculation","title":"Padlock Calculation - App Identity Specification","type":"extras"},{"doc":"The padlock cannot be presented by itself, because the digest used are one-way\ncryptographic hashes. Therefore, the client must supply the id (used to find the\nclient definition on the server) and the nonce (because the nonce is generated\nby the client application and not known by the server). This will typically be\nprovided as a concatenated, base-64-encoded string:\n\n**Ruby**:\n\n```ruby\nrequire 'base64'\n# Version 1\nBase64.urlsafe_encode64([id, nonce, padlock].join(':'))\n# Version 2+\nBase64.urlsafe_encode64([version, id, nonce, padlock].join(':'))\n```\n\n**Elixir**:\n\n```elixir\n[version, id, nonce, padlock]\n|> Enum.join(\":\")\n|> Base.urlsafe_encode64(padding: false)\n```\n\n**Swift**:\n\n```swift\nlet value = \"\\(version):\\(id):\\(nonce):\\(padlock)\"\nlet proof = Data(value.utf8).base64EncodedString()\n```\n\n**Typescript**:\n\n```typescript\nimport base64url from 'base64-url' // https://www.npmjs.com/package/base64-url\nconst parts = version === 1 ? [id, nonce, padlock] : [version, id, nonce, padlock]\nbase64url.encode(parts.join(':'))\n```\n\n**Java**:\n\n```java\nString value = version + \":\" + id + \":\" + nonce + \":\" + padlock.toString();\nbyte[] encodedHash = Base64.getEncoder().encode(encodeValue.getBytes());\nString proof = new String(encodedHash, \"UTF-8\");\n```\n\n[global id]: https://github.com/rails/globalid\n[rfc2119]: https://datatracker.ietf.org/doc/html/rfc2119\n[rfc3339]: https://datatracker.ietf.org/doc/html/rfc3339\n[§5.6]: https://tools.ietf.org/html/rfc3339#section-5.6\n[semantic versioning]: http://semver.org/\n[abnf]: https://www.rfc-editor.org/rfc/rfc2234.txt\n[gitguardian]: https://www.gitguardian.com\n[gitleaks]: https://gitleaks.io\n[github secret scanning]: https://docs.github.com/en/code-security/secret-scanning/about-secret-scanning","ref":"spec.html#padlock-presentation","title":"Padlock Presentation - App Identity Specification","type":"extras"},{"doc":"# Licence\n\nApp Identity for Elixir is copyright © 2022 Kinetic Commerce and contributors\nand is licensed under [Apache Licence, version 2.0][apache-licence-20].","ref":"Licence.md.html","title":"Licence","type":"extras"},{"doc":"All contributors **must** certify they are able and willing to provide their\ncontributions under the terms of this project's licenses with the certification\nof the [Developer Certificate of Origin (Version 1.1)][dco].\n\nSuch certification is provided by ensuring that the following line must be\nincluded as the last line of a commit message for every commit contributed:\n\n    Signed-off-by: FirstName LastName  \n\nThe `Signed-off-by` line can be automatically added by git with the `-s` or\n`--signoff` option on `git commit`:\n\n```sh\ngit commit --signoff\n```\n\n[apache-licence-20]: licences/APACHE-2.0.txt\n[dco]: licences/dco.txt","ref":"Licence.md.html#developer-certificate-of-origin","title":"Developer Certificate of Origin - Licence","type":"extras"},{"doc":"Apache License\nVersion 2.0, January 2004\nhttp://www.apache.org/licenses/\n\nTERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n1. Definitions.\n\n\"License\" shall mean the terms and conditions for use, reproduction, and\ndistribution as defined by Sections 1 through 9 of this document.\n\n\"Licensor\" shall mean the copyright owner or entity authorized by the copyright\nowner that is granting the License.\n\n\"Legal Entity\" shall mean the union of the acting entity and all other entities\nthat control, are controlled by, or are under common control with that entity.\nFor the purposes of this definition, \"control\" means (i) the power, direct or\nindirect, to cause the direction or management of such entity, whether by\ncontract or otherwise, or (ii) ownership of fifty percent (50%) or more of the\noutstanding shares, or (iii) beneficial ownership of such entity.\n\n\"You\" (or \"Your\") shall mean an individual or Legal Entity exercising\npermissions granted by this License.\n\n\"Source\" form shall mean the preferred form for making modifications, including\nbut not limited to software source code, documentation source, and configuration\nfiles.\n\n\"Object\" form shall mean any form resulting from mechanical transformation or\ntranslation of a Source form, including but not limited to compiled object code,\ngenerated documentation, and conversions to other media types.\n\n\"Work\" shall mean the work of authorship, whether in Source or Object form, made\navailable under the License, as indicated by a copyright notice that is included\nin or attached to the work (an example is provided in the Appendix below).\n\n\"Derivative Works\" shall mean any work, whether in Source or Object form, that\nis based on (or derived from) the Work and for which the editorial revisions,\nannotations, elaborations, or other modifications represent, as a whole, an\noriginal work of authorship. For the purposes of this License, Derivative Works\nshall not include works that remain separable from, or merely link (or bind by\nname) to the interfaces of, the Work and Derivative Works thereof.\n\n\"Contribution\" shall mean any work of authorship, including the original version\nof the Work and any modifications or additions to that Work or Derivative Works\nthereof, that is intentionally submitted to Licensor for inclusion in the Work\nby the copyright owner or by an individual or Legal Entity authorized to submit\non behalf of the copyright owner. For the purposes of this definition,\n\"submitted\" means any form of electronic, verbal, or written communication sent\nto the Licensor or its representatives, including but not limited to\ncommunication on electronic mailing lists, source code control systems, and\nissue tracking systems that are managed by, or on behalf of, the Licensor for\nthe purpose of discussing and improving the Work, but excluding communication\nthat is conspicuously marked or otherwise designated in writing by the copyright\nowner as \"Not a Contribution.\"\n\n\"Contributor\" shall mean Licensor and any individual or Legal Entity on behalf\nof whom a Contribution has been received by Licensor and subsequently\nincorporated within the Work.\n\n2. Grant of Copyright License.\n\nSubject to the terms and conditions of this License, each Contributor hereby\ngrants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free,\nirrevocable copyright license to reproduce, prepare Derivative Works of,\npublicly display, publicly perform, sublicense, and distribute the Work and such\nDerivative Works in Source or Object form.\n\n3. Grant of Patent License.\n\nSubject to the terms and conditions of this License, each Contributor hereby\ngrants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free,\nirrevocable (except as stated in this section) patent license to make, have\nmade, use, offer to sell, sell, import, and otherwise transfer the Work, where\nsuch license applies only to those patent claims licensable by such Contributor\nthat are necessarily infringed by their Contribution(s) alone or by combination\nof their Contribution(s) with the Work to which such Contribution(s) was\nsubmitted. If You institute patent litigation against any entity (including\na cross-claim or counterclaim in a lawsuit) alleging that the Work or\na Contribution incorporated within the Work constitutes direct or contributory\npatent infringement, then any patent licenses granted to You under this License\nfor that Work shall terminate as of the date such litigation is filed.\n\n4. Redistribution.\n\nYou may reproduce and distribute copies of the Work or Derivative Works thereof\nin any medium, with or without modifications, and in Source or Object form,\nprovided that You meet the following conditions:\n\n- You must give any other recipients of the Work or Derivative Works a copy of\n  this License; and\n- You must cause any modified files to carry prominent notices stating that You\n  changed the files; and\n- You must retain, in the Source form of any Derivative Works that You\n  distribute, all copyright, patent, trademark, and attribution notices from the\n  Source form of the Work, excluding those notices that do not pertain to any\n  part of the Derivative Works; and\n- If the Work includes a \"NOTICE\" text file as part of its distribution, then\n  any Derivative Works that You distribute must include a readable copy of the\n  attribution notices contained within such NOTICE file, excluding those notices\n  that do not pertain to any part of the Derivative Works, in at least one of\n  the following places: within a NOTICE text file distributed as part of the\n  Derivative Works; within the Source form or documentation, if provided along\n  with the Derivative Works; or, within a display generated by the Derivative\n  Works, if and wherever such third-party notices normally appear. The contents\n  of the NOTICE file are for informational purposes only and do not modify the\n  License. You may add Your own attribution notices within Derivative Works that\n  You distribute, alongside or as an addendum to the NOTICE text from the Work,\n  provided that such additional attribution notices cannot be construed as\n  modifying the License.\n\nYou may add Your own copyright statement to Your modifications and may provide\nadditional or different license terms and conditions for use, reproduction, or\ndistribution of Your modifications, or for any such Derivative Works as a whole,\nprovided Your use, reproduction, and distribution of the Work otherwise complies\nwith the conditions stated in this License.\n\n5. Submission of Contributions.\n\nUnless You explicitly state otherwise, any Contribution intentionally submitted\nfor inclusion in the Work by You to the Licensor shall be under the terms and\nconditions of this License, without any additional terms or conditions.\nNotwithstanding the above, nothing herein shall supersede or modify the terms of\nany separate license agreement you may have executed with Licensor regarding\nsuch Contributions.\n\n6. Trademarks.\n\nThis License does not grant permission to use the trade names, trademarks,\nservice marks, or product names of the Licensor, except as required for\nreasonable and customary use in describing the origin of the Work and\nreproducing the content of the NOTICE file.\n\n7. Disclaimer of Warranty.\n\nUnless required by applicable law or agreed to in writing, Licensor provides the\nWork (and each Contributor provides its Contributions) on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied,\nincluding, without limitation, any warranties or conditions of TITLE,\nNON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are\nsolely responsible for determining the appropriateness of using or\nredistributing the Work and assume any risks associated with Your exercise of\npermissions under this License.\n\n8. Limitation of Liability.\n\nIn no event and under no legal theory, whether in tort (including negligence),\ncontract, or otherwise, unless required by applicable law (such as deliberate\nand grossly negligent acts) or agreed to in writing, shall any Contributor be\nliable to You for damages, including any direct, indirect, special, incidental,\nor consequential damages of any character arising as a result of this License or\nout of the use or inability to use the Work (including but not limited to\ndamages for loss of goodwill, work stoppage, computer failure or malfunction, or\nany and all other commercial damages or losses), even if such Contributor has\nbeen advised of the possibility of such damages.\n\n9. Accepting Warranty or Additional Liability.\n\nWhile redistributing the Work or Derivative Works thereof, You may choose to\noffer, and charge a fee for, acceptance of support, warranty, indemnity, or\nother liability obligations and/or rights consistent with this License. However,\nin accepting such obligations, You may act only on Your own behalf and on Your\nsole responsibility, not on behalf of any other Contributor, and only if You\nagree to indemnify, defend, and hold each Contributor harmless for any liability\nincurred by, or claims asserted against, such Contributor by reason of your\naccepting any such warranty or additional liability.\n\nEND OF TERMS AND CONDITIONS","ref":"APACHE-2.0.txt.html","title":"Apache Licence, version 2.0","type":"extras"},{"doc":"Developer Certificate of Origin\nVersion 1.1\n\nCopyright (C) 2004, 2006 The Linux Foundation and its contributors.\n\nEveryone is permitted to copy and distribute verbatim copies of this\nlicense document, but changing it is not allowed.\n\n\nDeveloper's Certificate of Origin 1.1\n\nBy making a contribution to this project, I certify that:\n\n(a) The contribution was created in whole or in part by me and I\n    have the right to submit it under the open source license\n    indicated in the file; or\n\n(b) The contribution is based upon previous work that, to the best\n    of my knowledge, is covered under an appropriate open source\n    license and I have the right under that license to submit that\n    work with modifications, whether created in whole or in part\n    by me, under the same open source license (unless I am\n    permitted to submit under a different license), as indicated\n    in the file; or\n\n(c) The contribution was provided directly to me by some other\n    person who certified (a), (b) or (c) and I have not modified\n    it.\n\n(d) I understand and agree that this project and the contribution\n    are public and that a record of the contribution (including all\n    personal information I submit with it, including my sign-off) is\n    maintained indefinitely and may be redistributed consistent with\n    this project or the open source license(s) involved.","ref":"dco.txt.html","title":"Developer Certificate of Origin","type":"extras"}]}